@@ -1,122 +1,35 @@
 package template
 
 import (
-	"bytes"
-	"errors"
+	"fmt"
 	"reflect"
 	"strings"
-	"text/template"
+	"time"
 
-	"github.com/jandedobbeleer/oh-my-posh/src/platform"
-	"github.com/jandedobbeleer/oh-my-posh/src/regex"
-)
-
-const (
-	// Errors to show when the template handling fails
-	InvalidTemplate   = "invalid template text"
-	IncorrectTemplate = "unable to create text based on template"
-
-	globalRef = ".$"
-)
-
-var (
-	knownVariables = []string{
-		"Root",
-		"PWD",
-		"Folder",
-		"Shell",
-		"ShellVersion",
-		"UserName",
-		"HostName",
-		"Env",
-		"Data",
-		"Code",
-		"OS",
-		"WSL",
-		"Segments",
-		"Templates",
-		"PromptCount",
-		"Var",
-	}
+	"github.com/jandedobbeleer/oh-my-posh/src/log"
 )
 
 type Text struct {
-	Template        string
-	Context         any
-	Env             platform.Environment
-	TemplatesResult string
-}
-
-type Data any
-
-type Context struct {
-	*platform.TemplateCache
-
-	// Simple container to hold ANY object
-	Data
-	Templates string
-}
-
-func (c *Context) init(t *Text) {
-	c.Data = t.Context
-	c.Templates = t.TemplatesResult
-	if cache := t.Env.TemplateCache(); cache != nil {
-		c.TemplateCache = cache
-		return
-	}
+	Context  Data
+	Template string
 }
 
 func (t *Text) Render() (string, error) {
-	t.Env.DebugF("Rendering template: %s", t.Template)
+	defer log.Trace(time.Now(), t.Template)
+
 	if !strings.Contains(t.Template, "{{") || !strings.Contains(t.Template, "}}") {
 		return t.Template, nil
 	}
-	t.cleanTemplate()
-	tmpl, err := template.New(t.Template).Funcs(funcMap()).Parse(t.Template)
-	if err != nil {
-		t.Env.Error(err)
-		return "", errors.New(InvalidTemplate)
-	}
-	context := &Context{}
-	context.init(t)
-	buffer := new(bytes.Buffer)
-	defer buffer.Reset()
-	err = tmpl.Execute(buffer, context)
-	if err != nil {
-		t.Env.Error(err)
-		msg := regex.FindNamedRegexMatch(`at (?P<MSG><.*)$`, err.Error())
-		if len(msg) == 0 {
-			return "", errors.New(IncorrectTemplate)
-		}
-		return "", errors.New(msg["MSG"])
-	}
-	text := buffer.String()
-	// issue with missingkey=zero ignored for map[string]any
-	// https://github.com/golang/go/issues/24963
-	text = strings.ReplaceAll(text, "<no value>", "")
-	return text, nil
+
+	t.patchTemplate()
+
+	renderer := renderPool.Get().(*renderer)
+	defer renderer.release()
+
+	return renderer.execute(t)
 }
 
-func (t *Text) cleanTemplate() {
-	isKnownVariable := func(variable string) bool {
-		variable = strings.TrimPrefix(variable, ".")
-		splitted := strings.Split(variable, ".")
-		if len(splitted) == 0 {
-			return true
-		}
-		variable = splitted[0]
-		// check if alphanumeric
-		if !regex.MatchString(`^[a-zA-Z0-9]+$`, variable) {
-			return true
-		}
-		for _, b := range knownVariables {
-			if variable == b {
-				return true
-			}
-		}
-		return false
-	}
-
+func (t *Text) patchTemplate() {
 	fields := make(fields)
 	fields.init(t.Context)
 
@@ -133,10 +46,12 @@ func (t *Text) cleanTemplate() {
 				inTemplate = false
 			}
 		}
+
 		if !inTemplate {
 			result += string(char)
 			continue
 		}
+
 		switch char {
 		case '.':
 			var lastChar rune
@@ -157,26 +72,32 @@ func (t *Text) cleanTemplate() {
 				result += string(char)
 				continue
 			}
-			// end of a variable, needs to be appended
-			if !isKnownVariable(property) { //nolint: gocritic
-				result += ".Data" + property
-			} else if strings.HasPrefix(property, ".Segments") && !strings.HasSuffix(property, ".Contains") {
+
+			switch {
+			case strings.HasPrefix(property, ".Segments") && !strings.HasSuffix(property, ".Contains"):
 				// as we can't provide a clean way to access the list
 				// of segments, we need to replace the property with
 				// the list of segments so they can be accessed directly
-				property = strings.Replace(property, ".Segments", ".Segments.SimpleMap", 1)
+				property = strings.Replace(property, ".Segments", ".Segments.ToSimple", 1)
 				result += property
-			} else {
+			case strings.HasPrefix(property, ".Env."):
+				// we need to replace the property with the getEnv function
+				// so we can access the environment variables directly
+				property = strings.TrimPrefix(property, ".Env.")
+				result += fmt.Sprintf(`(call .Getenv "%s")`, property)
+			default:
 				// check if we have the same property in Data
 				// and replace it with the Data property so it
 				// can take precedence
 				if fields.hasField(property) {
 					property = ".Data" + property
 				}
+
 				// remove the global reference so we can use it directly
 				property = strings.TrimPrefix(property, globalRef)
 				result += property
 			}
+
 			property = ""
 			result += string(char)
 			inProperty = false
@@ -203,10 +124,48 @@ func (f *fields) init(data any) {
 	val := reflect.TypeOf(data)
 	switch val.Kind() { //nolint:exhaustive
 	case reflect.Struct:
+		name := val.Name()
+
+		// ignore the base struct
+		if name == "base" {
+			return
+		}
+
+		// check if we already know the fields of this struct
+		if kf, OK := knownFields.Get(name); OK {
+			for key := range kf.(fields) {
+				(*f)[key] = true
+			}
+			return
+		}
+
+		// Get struct fields and check embedded types
 		fieldsNum := val.NumField()
 		for i := 0; i < fieldsNum; i++ {
-			(*f)[val.Field(i).Name] = true
+			field := val.Field(i)
+			(*f)[field.Name] = true
+
+			// If this is an embedded field, get its methods too
+			if !field.Anonymous {
+				continue
+			}
+
+			embeddedType := field.Type
+
+			// Recursively check if the embedded type is also a struct
+			if embeddedType.Kind() == reflect.Struct {
+				f.init(reflect.New(embeddedType).Elem().Interface())
+			}
 		}
+
+		// Get pointer methods
+		ptrType := reflect.PointerTo(val)
+		methodsNum := ptrType.NumMethod()
+		for i := 0; i < methodsNum; i++ {
+			(*f)[ptrType.Method(i).Name] = true
+		}
+
+		knownFields.Set(name, *f)
 	case reflect.Map:
 		m, ok := data.(map[string]any)
 		if !ok {
@@ -222,6 +181,11 @@ func (f *fields) init(data any) {
 
 func (f fields) hasField(field string) bool {
 	field = strings.TrimPrefix(field, ".")
+
+	// get the first part of the field
+	splitted := strings.Split(field, ".")
+	field = splitted[0]
+
 	_, ok := f[field]
 	return ok
 }
